@@ -36,8 +36,8 @@ static gchar* history_magics[]={
 																NULL,
 };
 
-/*#define HISTORY_FILE0 HISTORY_FILE */
-#define HISTORY_FILE0 "parcellite/hist.test"
+#define HISTORY_FILE0 HISTORY_FILE 
+/*#define HISTORY_FILE0 "parcellite/hist.test" */
 
 /***************************************************************************/
 /** Pass in the text via the struct. We assume len is correct, and BYTE based,
@@ -52,10 +52,10 @@ glong validate_utf8_text(gchar *text, glong len)
 		return 0;
 	text[len]=0;
 	if(FALSE == g_utf8_validate(text,-1,&valid)) {
-		g_printf("Truncating invalid utf8 text entry: ");
+		g_fprintf(stderr,"Truncating invalid utf8 text entry: ");
 		len=valid-text;
 		text[len]=0;
-		g_printf("'%s'\n",text);
+		g_fprintf(stderr,"'%s'\n",text);
 	}
 	return len;
 }
@@ -95,8 +95,9 @@ void read_history_old ()
 			}else{
 				c->text[size] = 0;
 				c->len=validate_utf8_text(c->text,c->len);
-	      /* Prepend item and read next size */
-	      history_list = g_list_prepend(history_list, c);
+				if(0 != c->len) /* Prepend item and read next size */
+	      	history_list = g_list_prepend(history_list, c);
+				else g_free(c);
 			}
     }
     /* Close file and reverse the history to normal */
@@ -181,11 +182,11 @@ void read_history ()
     /* Read the magic*/
     guint32 size=1, end;
 		if (fread(magic,HISTORY_MAGIC_SIZE , 1, history_file) != 1){
-			g_printf("No magic! Assume no history.\n");
+			g_fprintf(stderr,"No magic! Assume no history.\n");
 			goto done;
 		}
     if(HISTORY_VERSION !=check_magic(magic)){
-			g_printf("Assuming old history style. Read and convert.\n");
+			g_fprintf(stderr,"Assuming old history style. Read and convert.\n");
 			/*g_printf("TODO! History version not matching!!Discarding history.\n"); */
 			g_free(magic);
 			fclose(history_file);
@@ -205,19 +206,20 @@ void read_history ()
 			end=size-(sizeof(struct history_item)+4);
       
       if (fread(c, sizeof(struct history_item), 1, history_file) !=1)
-      	g_printf("history_read: Invalid type!");
+      	g_fprintf(stderr,"history_read: Invalid type!");
 			if(c->len != end)
-				g_printf("len check: invalid: ex %d got %d\n",end,c->len);
+				g_fprintf(stderr,"len check: invalid: ex %d got %d\n",end,c->len);
 			/* Read item and add ending character */
 			if (fread(&c->text,end,1,history_file) != 1){
 				c->text[end] = 0;
-				g_printf("history_read: Invalid text!\n'%s'\n",c->text);
+				g_fprintf(stderr,"history_read: Invalid text!\n'%s'\n",c->text);
 			}	else {
 				c->text[end] = 0;
 				c->len=validate_utf8_text(c->text,c->len);
-				if(dbg) g_printf("len %d type %d '%s'\n",c->len,c->type,c->text); 
-	      /* Prepend item and read next size */
-	      history_list = g_list_prepend(history_list, c);	
+				if(dbg) g_fprintf(stderr,"len %d type %d '%s'\n",c->len,c->type,c->text); 
+				if(0 != c->len) /* Prepend item and read next size */
+	      	history_list = g_list_prepend(history_list, c);
+				else g_free(c);
 			}
       
     }
@@ -281,10 +283,13 @@ void save_history()
 			c=(struct history_item *)element->data;
 			/**write total len  */
 			/**write total len  */
-			len=c->len+sizeof(struct history_item)+4;
-			fwrite(&len,4,1,history_file);
-			fwrite(c,sizeof(struct history_item),1,history_file);
-			fwrite(c->text,c->len,1,history_file);
+			if(c->len >0){
+				len=c->len+sizeof(struct history_item)+4;
+				fwrite(&len,4,1,history_file);
+				fwrite(c,sizeof(struct history_item),1,history_file);
+				fwrite(c->text,c->len,1,history_file);	
+			}
+			
     }
     /* Write 0 to indicate end of file */
     gint end = 0;
@@ -333,8 +338,8 @@ gint is_duplicate(gchar* item, int mode, gint *flags)
 						*flags=c->flags;
 					}
 					/*g_printf("Freeing 0x%02X '%s'\n",c->flags,c->text);  */
-					g_free(element->data);
-		      history_list = g_list_delete_link(history_list, element);
+					/** g_free(element->data);
+		      history_list = g_list_delete_link(history_list, element);*/
 				}
 				return i;
 	      break;
@@ -351,10 +356,12 @@ gint is_duplicate(gchar* item, int mode, gint *flags)
 void append_item(gchar* item, int checkdup)
 {
 	gint flags=0,node=-1;
+	GList *element=NULL;
+	struct history_item *c=NULL;
 	if(NULL == item)
 		return;
 	g_mutex_lock(hist_lock);
-/**delete if HIST_DEL flag is set.  */	
+/**delete if HIST_DEL flag is set.  */
 	if( checkdup & HIST_CHECKDUP){
 		node=is_duplicate(item, checkdup & HIST_DEL, &flags);
 		/*g_printf("isd done "); */
@@ -364,18 +371,23 @@ void append_item(gchar* item, int checkdup)
 				return;	
 		}
 	}
-	
-	struct history_item *c;
-	if(NULL == (c=new_clip_item(CLIP_TYPE_TEXT,strlen(item),item)) )
-		return;
-	if(node > -1 && (checkdup & HIST_KEEP_FLAGS) ){
-		c->flags=flags;
-		/*g_printf("Restoring 0x%02X '%s'\n",c->flags,c->text);  */
+	if (-1 != node ){/**we found a duplicate in the history, remove, then re-add existing  */
+		element=g_list_nth(history_list,node);
+		history_list=g_list_remove_link(history_list,element);
+	}else{ /**not found   */
+		if(NULL == (c=new_clip_item(CLIP_TYPE_TEXT,strlen(item),item)) )
+			return;
+		if(node > -1 && (checkdup & HIST_KEEP_FLAGS) ){
+			c->flags=flags;
+			/*g_printf("Restoring 0x%02X '%s'\n",c->flags,c->text);  */
+		}		
 	}
+	
+
 		
 	/*g_printf("Append '%s'\n",item); */
    /* Prepend new item */
-  history_list = g_list_prepend(history_list, c);
+  history_list = g_list_prepend(history_list, NULL ==element?c:element->data);
   /* Shorten history if necessary */
   truncate_history();
 	g_mutex_unlock(hist_lock);
@@ -518,6 +530,7 @@ void history_save_as(GtkMenuItem *menu_item, gpointer user_data)
 				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 				      NULL);
+	gtk_window_set_icon((GtkWindow*)dialog, gtk_widget_render_icon(dialog, GTK_STOCK_SAVE_AS, -1, NULL));
 	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
 /*	if (user_edited_a_new_document)  { */
 	    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), g_get_home_dir());
