@@ -50,6 +50,12 @@ correct behavior:
    work with many clients, and they require data to be 
    copied to the X server. Therefore clients should avoid 
    using cut buffers and use only selections.
+   
+We can monitor ownership change with:
+// This callback is invoked when the clipboard owner changes.
+void handle_owner_change(GtkClipboard *clipboard, GdkEvent *event,  gpointer data){	}
+}
+g_signal_connect(clipboard, "owner-change",  G_CALLBACK(handle_owner_change), NULL);
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,7 +82,7 @@ correct behavior:
 
 GtkWidget *hmenu;
 /* Uncomment the next line to print a debug trace. */
-/*#define DEBUG    */
+/*#define DEBUG     */
 
 #ifdef DEBUG
 #  define TRACE(x) x
@@ -122,6 +128,7 @@ static int cmd_mode=CMODE_ALL; /**both clipboards  */
 
 #define EDIT_MODE_USE_RIGHT_CLICK 1 /**used in edit dialog creation to determine behaviour. 
                                     If this is set, it will edit the entry, and replace it in the history.  */
+#define EDIT_MODE_RC_EDIT_SET 2 /**used to   */                                    
 
 /**protos in this file  */
 void create_app_indicator(void);
@@ -235,7 +242,7 @@ gchar *_update_clipboard (GtkClipboard *clip, gchar *n, gchar **old, int set)
 }
 
 /***************************************************************************/
-/** .
+/** This checks to see if ANY content exists (i.e. like images).
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
@@ -248,10 +255,8 @@ gboolean content_exists(GtkClipboard *clip)
 	return contents;
 }
 
-
 /***************************************************************************/
-/** This DOES NOT WORK!! WTH??.
-Fix is a bit hacky.
+/** 
 \n\b Arguments:
 \n\b Returns:	 clipboard contents, to be freed with g_free, or NULL if empty.
 ****************************************************************************/
@@ -265,8 +270,25 @@ gchar *is_clipboard_empty(GtkClipboard *clip)
   g_free(targets);
 	if(TRUE == contents || count >0)
 		return 0;*/
-	if(TRUE == gtk_clipboard_wait_is_text_available(clip))
-		return(gtk_clipboard_wait_for_text(clip));
+#if 1
+	 if(TRUE == gtk_clipboard_wait_is_text_available(clip))
+   	return(gtk_clipboard_wait_for_text(clip));
+#else	
+	/**attempt to fix  bug 87, Error converting selection from UTF8_STRING  
+	Go http://www.cl.cam.ac.uk/~mgk25/ucs/examples/UTF-8-test.txt
+	line 2.3.4 will reproduce this using FireFox.	*/
+	if(TRUE == gtk_clipboard_wait_is_text_available(clip)){
+		gchar *x=gtk_clipboard_wait_for_text(clip);
+		if(FALSE == g_utf8_validate(x,-1,NULL))
+			gtk_clipboard_set_text(clip,"_BAD_UTF8_",-1);
+		/** ulong l1, l2;
+		l1=strlen*/
+		/**if NULL == x, we have a problem with whatever is in the clipboard.  */
+		/*if(NULL ==x) */
+			
+		return(x);
+	}
+#endif		
 	return NULL;
 }
 
@@ -323,15 +345,15 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 	}	
 	/*g_printf("BS=0x%02X ",button_state); */
 	if( H_MODE_IGNORE == mode){	/**ignore processing and just put it on the clip.  */
-		DTRACE(g_printf("%sJustSet '%s'\n",clip==clipboard?"CLI":"PRI",intext)); 
+		DTRACE(g_fprintf(stderr,"%sJustSet '%s'\n",clip==clipboard?"CLI":"PRI",intext)); 
 		_update_clipboard(clip,intext,NULL,1);
 		/*gtk_clipboard_set_text(clip, intext, -1); */
 		return intext;
 	}
 	if(H_MODE_LIST == mode && 0 != p_strcmp(intext,*existing)){ /**just set clipboard contents. Already in list  */
-		DTRACE(g_printf("%sInList '%s' ex '%s'\n",clip==clipboard?"CLI":"PRI",intext,*existing)); 
+		DTRACE(g_fprintf(stderr,"%sInList '%s' ex '%s'\n",clip==clipboard?"CLI":"PRI",intext,*existing)); 
 		last=_update_clipboard(clip,intext,existing,1);
-		if(NULL != last){/**maintain persistence, if set  */
+		if( NULL != last){/**maintain persistence, if set  */
 			append_item(last,get_pref_int32("current_on_top")?HIST_DEL|HIST_CHECKDUP|HIST_KEEP_FLAGS:0);
 		}
 		goto done;
@@ -339,19 +361,20 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 	/**check for lost contents and restore if lost */
 	changed=is_clipboard_empty(clip);
 	if(NULL != changed){
-		validate_utf8_text(changed,strlen(changed));
+		if(0 == validate_utf8_text(changed,strlen(changed)))
+			goto done;
 	}
 	
 	if(NULL != *existing && NULL == changed && 1 == get_pref_int32("restore_empty")) {
-		DTRACE(g_printf("%sclp empty, ",clip==clipboard?"CLI":"PRI"));
+		DTRACE(g_fprintf(stderr,"%sclp empty, ",clip==clipboard?"CLI":"PRI"));
 		/* Only recover lost contents if there isn't any other type of content in the clipboard */
 		if (!content_exists(clip)) {
-			DTRACE(g_printf("set to '%s'\n",*existing));  
+			DTRACE(g_fprintf(stderr,"set to '%s'\n",*existing));  
 			_update_clipboard(clip, *existing,NULL,1);
     	/*gtk_clipboard_set_text(clip, *existing, -1); */
 			last=*existing;
 		}	else
-			DTRACE(g_printf("Left Null\n"));  
+			DTRACE(g_fprintf(stderr,"Left Null\n"));  
 		return *existing;
   }
 	if(NULL == changed)
@@ -362,12 +385,12 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 		g_free(changed);                    /**no change, do nothing  */
 		changed=NULL;
 	}	else {
-		DTRACE(g_printf("%sclp changed: ex '%s' is '%s' - ",clip==clipboard?"CLI":"PRI",*existing,changed)); 
+		DTRACE(g_fprintf(stderr,"%sclp changed: ex '%s' is '%s' - ",clip==clipboard?"CLI":"PRI",*existing,changed)); 
 		if(NULL != (processed=process_new_item(clip,changed)) ){ 
 			/**only check processed/changed. No need to update this clip, since the text is already there.  */
 			 if(0 == p_strcmp(processed,changed)) set=0;
 			else set=1; 
-			DTRACE(g_printf("set=%d. p='%s'\n",set,processed));
+			DTRACE(g_fprintf(stderr,"set=%d. p='%s'\n",set,processed));
 			/*set=1; */ /** Always set the text.*/
 			last=_update_clipboard(clip,processed,existing,set);
 		}else {/**restore clipboard - new item is binary/garbage/empty */
@@ -380,7 +403,7 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 			}else 
 				d=*existing;
 			if(NULL != d){
-				DTRACE(g_printf("\n%srestore clp '%s', ex='%s'\n",clip==clipboard?"CLI":"PRI",d,*existing)); 
+				DTRACE(g_fprintf(stderr,"\n%srestore clp '%s', ex='%s'\n",clip==clipboard?"CLI":"PRI",d,*existing)); 
 				last=_update_clipboard(clip,d,existing,1);
 			}
 				
@@ -583,7 +606,7 @@ static void *execute_action(void *command)
   gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Executing action..."));
   }
   if(system((gchar*)command))
-  	g_print("sytem command '%s' failed\n",(gchar *)command);
+  	g_fprintf(stderr,"sytem command '%s' failed\n",(gchar *)command);
   if (!have_appindicator &&show_icon) {
 	gtk_status_icon_set_from_icon_name((GtkStatusIcon*)status_icon, PARCELLITE_ICON);
   gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Clipboard Manager"));
@@ -616,9 +639,9 @@ static void action_selected(GtkButton *button, gpointer user_data)
   }
   /* Insert clipboard into command (user_data), and prepare it for execution */
   gchar* clipboard_text = gtk_clipboard_wait_for_text(clipboard);
-	g_print("Got cmd '%s', text '%s'->",(gchar *)user_data,clipboard_text);fflush(NULL);  
+	g_fprintf(stderr,"Got cmd '%s', text '%s'->",(gchar *)user_data,clipboard_text);fflush(NULL);  
 	gchar* command=g_strdup_printf((gchar *)user_data,clipboard_text);
-	g_print(" '%s'\n",command);fflush(NULL);  
+	g_fprintf(stderr," '%s'\n",command);fflush(NULL);  
   g_free(clipboard_text);
   g_free(user_data);
   gchar* shell_command = g_shell_quote(command);
@@ -630,7 +653,7 @@ static void action_selected(GtkButton *button, gpointer user_data)
   GPid pid;
   gchar **argv;
   g_shell_parse_argv(cmd, NULL, &argv, NULL);
-	g_print("cmd '%s' argv '%s' '%s' '%s'\n",cmd,argv[1],argv[2],argv[3]);  
+	g_fprintf(stderr,"cmd '%s' argv '%s' '%s' '%s'\n",cmd,argv[1],argv[2],argv[3]);  
   g_free(cmd);
   g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
   g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
@@ -656,38 +679,51 @@ static void edit_selected(GtkMenuItem *menu_item, gpointer user_data)
 	struct history_info *h=(struct history_info*)user_data;
 	GList* element=NULL;
 	struct history_item *c=NULL;
+	if(NULL ==h)
+		return;
+	/*g_fprintf(stderr,"edit_selected call\n");  */
   /* This helps prevent multiple instances */
-  if (!gtk_grab_get_current())  {
-	  gchar* current_clipboard_text;
+  if (!gtk_grab_get_current() ||h->wi.tmp1&EDIT_MODE_RC_EDIT_SET)  {
+	  gchar* current_clipboard_text=NULL;
+		/*g_fprintf(stderr,"current..."); */
     /* Create clipboard buffer and set its text */
     GtkTextBuffer* clipboard_buffer = gtk_text_buffer_new(NULL);
-		if(h->wi.tmp1&EDIT_MODE_USE_RIGHT_CLICK){/**use index as pointer to text  */
+		if(h->wi.index != -1){/**use index as pointer to text  */
 			element = g_list_nth(history_list, h->wi.index);
-			if(NULL !=element){
-				c=(struct history_item *)(element->data);
-				current_clipboard_text=p_strdup(c->text);
-				/*g_printf("%s ",c->text); */
+			if(NULL == element){
+				g_fprintf(stderr,"edit_selected: element is NULL\n");
+				return;
 			}
+			c=(struct history_item *)(element->data);
+			if(NULL == c){
+				g_fprintf(stderr,"edit_selected: element->data is NULL\n");
+				return;
+			}
+			current_clipboard_text=p_strdup(c->text);
+			/*g_fprintf(stderr,"%s ",c->text);  */
+			
 		}	else{
 			h->wi.tmp1=0;
-			if( NULL != h->element_text){
+			if( NULL != h->element_text){/**this case should never happen  */
+				g_fprintf(stderr,"Oops. shouldn't be here\n");
 				current_clipboard_text=p_strdup(h->element_text);
 				
 			}else{
+				g_fprintf(stderr,"List Empty. Grab clipboard.\n");
 				current_clipboard_text = gtk_clipboard_wait_for_text(clipboard);
 			}	
 		}
 		
     
-    if (current_clipboard_text != NULL)
-    {
-			TRACE(g_print("Got '%s'\n",current_clipboard_text));
+    if (current_clipboard_text != NULL)   {
+		/*g_fprintf(stderr,"Got '%s'\n",current_clipboard_text); */
+			TRACE(g_fprintf(stderr,"Got '%s'\n",current_clipboard_text));
       gtk_text_buffer_set_text(clipboard_buffer, current_clipboard_text, -1);
     }	 else	 {
-			g_printf("NULL text to edit. Nothing to do.\n");
+			g_fprintf(stderr,"NULL text to edit. Nothing to do.\n");
 			return;
 		}
-    	
+    /*g_fprintf(stderr,"cr dialog\n");	 */
     
     /* Create the dialog */
     GtkWidget* dialog = gtk_dialog_new_with_buttons(_("Editing Clipboard"), NULL,
@@ -696,7 +732,7 @@ static void edit_selected(GtkMenuItem *menu_item, gpointer user_data)
                                                     GTK_STOCK_OK,           GTK_RESPONSE_ACCEPT, NULL);
     
     gtk_window_set_default_size((GtkWindow*)dialog, 450, 300);
-    gtk_window_set_icon((GtkWindow*)dialog, gtk_widget_render_icon(dialog, GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU, NULL));
+    gtk_window_set_icon((GtkWindow*)dialog, gtk_widget_render_icon(dialog, GTK_STOCK_EDIT, -1, NULL));
     
     /* Build the scrolled window with the text view */
     GtkWidget* scrolled_window = gtk_scrolled_window_new((GtkAdjustment*) gtk_adjustment_new(0, 0, 0, 0, 0, 0),
@@ -716,30 +752,67 @@ static void edit_selected(GtkMenuItem *menu_item, gpointer user_data)
 		
     if (gtk_dialog_run((GtkDialog*)dialog) == GTK_RESPONSE_ACCEPT) {
 			GtkTextIter start, end;
+			guint32 slen;
+			gchar *ntext=NULL;
       gtk_text_buffer_get_start_iter(clipboard_buffer, &start);
       gtk_text_buffer_get_end_iter(clipboard_buffer, &end);
       gchar* new_clipboard_text = gtk_text_buffer_get_text(clipboard_buffer, &start, &end, TRUE);
-			
-			if(h->wi.tmp1&EDIT_MODE_USE_RIGHT_CLICK){/**save changes to the history h.wi.index has the element */
-				struct history_item *n=new_clip_item(c->type, strlen(new_clipboard_text), new_clipboard_text);
-				n->flags=c->flags;
+			slen=strlen(new_clipboard_text);
+			if(0 == slen ){ /**just delete history entry, and set next in order to clipboard  */
+				if(NULL == element){/**just clear clipboard? or FIXME: is there a way to determine the element?  */
+					/*gtk_clipboard_set_text(clipboard,"",0); */
+					gtk_clipboard_clear(clipboard);
+					goto finish;
+				}
+				/*g_fprintf(stderr,"Freeing %p\n",element->data); */
 				g_free(element->data);
-				element->data=n;
-			}	else{ /* Save changes done to the clipboard */
-				update_clipboard(clipboard, new_clipboard_text, H_MODE_NEW);
+				if(element == history_list && NULL != element->next ){ /**set clipboard(s) to next entry  */
+					c=(struct history_item *)(element->next->data);	
+					if(NULL != c)
+						ntext=c->text;
+				}
+					
+	      history_list = g_list_delete_link(history_list, element);
+				if(NULL != ntext)/**set clipboards to next entry FIXME: Need logic here as to which clip(s) to update.  */
+	      	update_clipboards(ntext, H_MODE_LIST);
+			}else {/**Text is not blank  */
+				/*g_fprintf(stderr,"Try to add '%s'\n",new_clipboard_text); */
+				gint16 type;
+				if( NULL != c)
+					type=c->type;
+				else 
+					type=0;
+				/**save changes to the history - deallocate current entry, and add new entry \Uffffffff*/
+				/**FIXME: Need to filter this through existing & valid text? */
+				struct history_item *n=new_clip_item(type,slen, new_clipboard_text);
+				
+				if(NULL != c)
+					n->flags=c->flags;
+				if(NULL != element && NULL != element->data){
+					g_free(element->data);
+					history_list=g_list_delete_link(history_list, element);
+					history_list=g_list_prepend(history_list,c);
+				}else
+				
+				/**Does this cause crash since we are in the middle of a history window? 
+				Need to kill history menu too? 
+				What about setting up an indicator that on next tic, if history is not active, we update?
+				*/
+			   update_clipboards(n->text, H_MODE_NEW);
 			}
+finish:			
 			if(NULL != new_clipboard_text)
 				g_free(new_clipboard_text);	
     }
     gtk_widget_destroy(dialog);
     g_free(current_clipboard_text);
   }
-	else TRACE(g_print("gtk_grab_get_current returned !0\n"));
+	else TRACE(g_fprintf(stderr,"gtk_grab_get_current returned !0\n")); 
 
 }
 
 /***************************************************************************/
-/** .
+/** Only enabled when persistent history is enabled.
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
@@ -839,7 +912,7 @@ if (event->type == GDK_BUTTON_PRESS  &&  event->button == 3)
 void  history_item_right_click (struct history_info *h, GdkEventKey *e, gint index)
 {
   GtkWidget *menu, *menuitem;
-  menu = gtk_menu_new();
+  
 	struct history_item *c=NULL;
 	if(NULL !=h ){
 		GList* element = g_list_nth(history_list, h->wi.index);
@@ -847,11 +920,16 @@ void  history_item_right_click (struct history_info *h, GdkEventKey *e, gint ind
 			c=(struct history_item *)(element->data);
 			/*g_printf("%s ",c->text); */
 		}
+	} else{
+		g_fprintf(stderr,"h-i-r-c: h is NULL");
+		return;
 	}
-	menuitem = gtk_menu_item_new_with_label("Copy All to Clip");
-  g_signal_connect(menuitem, "activate", (GCallback) history_item_right_click_on_copy_all, (gpointer)h);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-	if(get_pref_int32("persistent_history")){
+	
+	if(get_pref_int32("persistent_history")) {
+		menu = gtk_menu_new();
+		menuitem = gtk_menu_item_new_with_label("Copy All to Clip");
+	  g_signal_connect(menuitem, "activate", (GCallback) history_item_right_click_on_copy_all, (gpointer)h);
+	  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 		/*g_printf("CreatehistR\n"); */
 		if(NULL != c){
 			if(c->flags & CLIP_TYPE_PERSISTENT)
@@ -871,17 +949,22 @@ void  history_item_right_click (struct history_info *h, GdkEventKey *e, gint ind
 		menuitem = gtk_menu_item_new_with_label("Cancel");
 	  g_signal_connect(menuitem, "activate", (GCallback) history_item_right_click_on_cancel, (gpointer)h);
 	  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);	
+	
+	  gtk_widget_show_all(menu);
+	
+	  /* Note: event can be NULL here when called from view_onPopupMenu;
+	   *  gdk_event_get_time() accepts a NULL argument */
+	   gtk_menu_popup(GTK_MENU(menu), h->menu, NULL, NULL, NULL,
+										0,
+	                 /*(e != NULL) ? ((GdkEventButton *)e)->button : 0, */
+	                 gdk_event_get_time((GdkEvent*)e));
+		/*gtk_widget_grab_focus(menu);  */	
+	}else if(get_pref_int32("rc_edit") ){ /**just edit the selected text  */
+		h->wi.tmp1|=EDIT_MODE_USE_RIGHT_CLICK|EDIT_MODE_RC_EDIT_SET;
+		edit_selected((GtkMenuItem *)NULL,(gpointer)h);
+		
 	}
-
-  gtk_widget_show_all(menu);
-
-  /* Note: event can be NULL here when called from view_onPopupMenu;
-   *  gdk_event_get_time() accepts a NULL argument */
-   gtk_menu_popup(GTK_MENU(menu), h->menu, NULL, NULL, NULL,
-									0,
-                 /*(e != NULL) ? ((GdkEventButton *)e)->button : 0, */
-                 gdk_event_get_time((GdkEvent*)e));
-	/*gtk_widget_grab_focus(menu);  */
+	
 }
 
 /* Called when Clear is selected from history menu */
@@ -919,25 +1002,23 @@ static void show_about_dialog(GtkMenuItem *menu_item, gpointer user_data)
   /* This helps prevent multiple instances */
   if (!gtk_grab_get_current())
   {
-    const gchar* authors[] = {"Gilberto \"Xyhthyx\" Miralla <xyhthyx@gmail.com>\nDoug Springer <gpib@rickyrockrat.net>", NULL};
+    const gchar* authors[] = {_("Gilberto \"Xyhthyx\" Miralla <xyhthyx@gmail.com>\nDoug Springer <gpib@rickyrockrat.net>"), NULL};
     const gchar* license =
       "This program is free software; you can redistribute it and/or modify\n"
       "it under the terms of the GNU General Public License as published by\n"
       "the Free Software Foundation; either version 3 of the License, or\n"
-      "(at your option) any later version.\n"
-      "\n"
+      "(at your option) any later version.\n\n"
       "This program is distributed in the hope that it will be useful,\n"
       "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
       "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-      "GNU General Public License for more details.\n"
-      "\n"
+      "GNU General Public License for more details.\n\n"
       "You should have received a copy of the GNU General Public License\n"
       "along with this program.  If not, see <http://www.gnu.org/licenses/>.";
     
     /* Create the about dialog */
     GtkWidget* about_dialog = gtk_about_dialog_new();
     gtk_window_set_icon((GtkWindow*)about_dialog,
-                        gtk_widget_render_icon(about_dialog, GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU, NULL));
+                        gtk_widget_render_icon(about_dialog, GTK_STOCK_ABOUT, -1, NULL));
     
     gtk_about_dialog_set_name((GtkAboutDialog*)about_dialog, "Parcellite");
     #ifdef HAVE_CONFIG_H	/**VER=555; sed "s#\(.*\)svn.*\".*#\1svn$VER\"#" config.h  */
@@ -949,15 +1030,14 @@ static void show_about_dialog(GtkMenuItem *menu_item, gpointer user_data)
     gtk_about_dialog_set_website((GtkAboutDialog*)about_dialog,
                                  "http://parcellite.sourceforge.net");
     
-    gtk_about_dialog_set_copyright((GtkAboutDialog*)about_dialog, "Copyright (C) 2007, 2008 Gilberto \"Xyhthyx\" Miralla\n"
-	   "Copyright (C) 2010-2013 Doug Springer");
+    gtk_about_dialog_set_copyright((GtkAboutDialog*)about_dialog, _("Copyright (C) 2007, 2008 Gilberto \"Xyhthyx\" Miralla\nCopyright (C) 2010-2013 Doug Springer"));
     gtk_about_dialog_set_authors((GtkAboutDialog*)about_dialog, authors);
     gtk_about_dialog_set_translator_credits ((GtkAboutDialog*)about_dialog,
                                              "Miloš Koutný <milos.koutny@gmail.com>\n"
                                              "Kim Jensen <reklamepost@energimail.dk>\n"
                                              "Eckhard M. Jäger <bart@neeneenee.de>\n"
                                              "Michael Stempin <mstempin@web.de>\n"
-                                             "Benjamin 'sphax3d' Danon <sphax3d@gmail.com>\n"
+                                             "Benjamin Danon <benjamin@sphax3d.org>\n" 
                                              "Németh Tamás <ntomasz@vipmail.hu>\n"
                                              "Davide Truffa <davide@catoblepa.org>\n"
                                              "Jiro Kawada <jiro.kawada@gmail.com>\n"
@@ -1203,7 +1283,7 @@ void postition_history(GtkMenu *menu,gint *x,gint *y,gboolean *push_in, gpointer
 				yy=1;
 			if(NULL !=x) *x=xx;
 			if(NULL !=y) *y=yy;	
-			TRACE(g_print("x=%d, y=%d\n",xx,yy));
+			TRACE(g_fprintf(stderr,"x=%d, y=%d\n",xx,yy));
 		}
 		
 	}
@@ -1213,7 +1293,7 @@ void postition_history(GtkMenu *menu,gint *x,gint *y,gboolean *push_in, gpointer
 /***************************************************************************/
 /** This handles events for the history menu, which is the parent of each
 item.
-\n\b Arguments:
+\n\b Arguments:	user is the struct history_info created in history menu.
 \n\b Returns: FALSE if key was not handled, TRUE if it was.
 You get two key presses if you return FALSE.
 ****************************************************************************/
@@ -1290,7 +1370,7 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 	/** if(user)
 	if(e->state & (GDK_SHIFT_MASK|GDK_LOCK_MASK) != e->state){
 		g_print("rfs to use mods\n");
-		TRACE(g_print("state is %X. Refusing to use mods\n",e->state));
+		TRACE(g_fprintf(stderr,"state is %X. Refusing to use mods\n",e->state));
 		return TRUE;
 	} have to use for _ and others*/
 	/**ignore Ctrl-Alt  */
@@ -1298,16 +1378,18 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 		return FALSE;
 	if( e->state & GDK_MOD1_MASK){/**alt key pressed  */
 		if(e->keyval == 'e'){
-			TRACE(g_print("Alt-E\n"));
+			TRACE(g_fprintf(stderr,"Alt-E\n"));
 			gtk_grab_remove(w);
 		  edit_selected((GtkMenuItem *)h, (gpointer)h);
+			return TRUE;
 		}
 			
 		else if(e->keyval == 'c'){
-			TRACE(g_print("Alt-C\n"));
+			TRACE(g_fprintf(stderr,"Alt-C\n"));
 			clear_selected(NULL, (gpointer)h); 
+			return TRUE;
 		}	else{
-			TRACE(g_print("Ignoring Alt-%c (0x%02x) state 0x%x",e->keyval,e->keyval,e->state));
+			TRACE(g_fprintf(stderr,"Ignoring Alt-%c (0x%02x) state 0x%x",e->keyval,e->keyval,e->state));
 		}
 		return FALSE;
 	}	/**end alt key pressed  */
@@ -1330,15 +1412,15 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 		return TRUE;
 	}	/**end backspace  */
 	if( e->keyval == 0xffe1 || e->keyval == 0xffe2){
-		printf("Ignoring key '%c' 0x%02x\n",e->keyval,e->keyval);
-		TRACE(g_print("Ignoring key '%c' 0x%02x\n",e->keyval,e->keyval));	
+		/*fprintf(stderr,"Ignoring key '%c' 0x%02x\n",e->keyval,e->keyval); */
+		TRACE(g_fprintf(stderr,"Ignoring key '%c' 0x%02x\n",e->keyval,e->keyval));	
 		return FALSE;
 	}
   if(e->keyval >= 0xff50 && e->keyval <= 0xff57) /**arrow keys, home,end,pgup,pgdwn  */
   	return FALSE;
   	
 	if(idx>=KBUF_SIZE){
-		TRACE(g_print("keys full\n"));
+		TRACE(g_fprintf(stderr,"keys full\n"));
 		return TRUE;
 	}
 	kstr[idx++]=e->keyval;
@@ -1368,7 +1450,7 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 						}
 						
 						if(!first ){
-							TRACE(g_print("Got cmp'%s'='%s'\n",kstr,l));
+							TRACE(g_fprintf(stderr,"Got cmp'%s'='%s'\n",kstr,l));
 							item=(GtkWidget *)children->data;
 							goto foundit;
 						}
@@ -1391,15 +1473,15 @@ foundit:
 	data should be a GtkMenuItem list, whose children are labels...
 	*/	 
 	/*str=(gchar *)gtk_label_get_text((GtkLabel *)gtk_bin_get_child((GtkBin*)children->data)); */
-	TRACE(g_print("Got '%c' 0x%02x, state 0x%02X",e->keyval,e->keyval,e->state));
+	TRACE(g_fprintf(stderr,"Got '%c' 0x%02x, state 0x%02X",e->keyval,e->keyval,e->state));
 	if(NULL !=item){
-		if(first){TRACE(g_print("First:"));}
-		/*if(last)TRACE(g_print("Last:")); */
-		TRACE(g_print("At Item '%s'",gtk_label_get_text((GtkLabel *)gtk_bin_get_child((GtkBin*)item))));
+		if(first){TRACE(g_fprintf(stderr,"First:"));}
+		/*if(last)TRACE(g_fprintf(stderr,"Last:")); */
+		TRACE(g_fprintf(stderr,"At Item '%s'",gtk_label_get_text((GtkLabel *)gtk_bin_get_child((GtkBin*)item))));
 		gtk_menu_shell_select_item((GtkMenuShell *)h->menu,(GtkWidget *)item);
 	}
 		
-	TRACE(g_print("\n"));
+	TRACE(g_fprintf(stderr,"\n"));
 	return TRUE;
 }	
 
@@ -1418,7 +1500,7 @@ void set_clipboard_text(struct history_info *h, GList *element)
 	if(NULL == find_h_item(h->delete_list,NULL,element)){	/**not in our delete list  */
 		/**make a copy of txt, because it gets freed and re-allocated.  */
 		txt=p_strdup(((struct history_item *)(element->data))->text);
-		DTRACE(("set_clip_text %s\n",txt));  
+		DTRACE(g_fprintf(stderr,"set_clip_text %s\n",txt));  
 		if(get_pref_int32("use_copy") )
 			update_clipboard(clipboard, txt, H_MODE_LIST);
 		if(get_pref_int32("use_primary"))
@@ -1446,7 +1528,7 @@ void set_clipboard_text(struct history_info *h, GList *element)
 			goto done;
 	/**from clipit 1.4.1 */
   cmd = g_strconcat("/bin/sh -c 'xdotool ", action, NULL);
-	g_printf("xdotool:'%s'\ntext:'%s'\n",cmd,txt);
+	g_fprintf(stderr,"xdotool:'%s'\ntext:'%s'\n",cmd,txt);
   GPid pid;
   gchar **argv;
   g_shell_parse_argv(cmd, NULL, &argv, NULL);
@@ -1512,8 +1594,9 @@ static gboolean my_item_event (GtkWidget *w,GdkEventKey *e, gpointer user)
 	      h->wi.event=e;
 	      h->wi.item=w;
 				h->wi.index=GPOINTER_TO_INT(user);
-				if(get_pref_int32("persistent_history"))
-		      history_item_right_click(h,e,GPOINTER_TO_INT(user));
+				/*g_fprintf(stderr,"Calling hist_itemRclk\n"); */
+		    history_item_right_click(h,e,GPOINTER_TO_INT(user));
+				
 			}
 			return TRUE;
 		}else if( 1 == b->button){
@@ -1610,6 +1693,7 @@ static gboolean show_history_menu(gpointer data)
 	h.histno=GPOINTER_TO_INT(data);/**persistent or normal history  */
 	h.change_flag=0;
 	h.element_text=NULL;
+	h.wi.index=-1;
   /**init our keystroke function  */
 	key_release_cb(NULL,NULL,NULL);
 	GList *element, *persistent=NULL;
@@ -1730,20 +1814,22 @@ static gboolean show_history_menu(gpointer data)
       if ((clipboard_temp) && (p_strcmp(hist_text, clipboard_temp) == 0))
       {
         gchar* bold_text = g_markup_printf_escaped("<b>%s</b>", string->str);
-			  if( NULL == bold_text) g_printf("NulBMKUp:'%s'\n",string->str);
+			  if( NULL == bold_text) g_fprintf(stderr,"NulBMKUp:'%s'\n",string->str);
         gtk_label_set_markup((GtkLabel*)item_label, bold_text);
         g_free(bold_text);
         h.clip_item=menu_item;
 				h.element_text=hist_text;
+			  h.wi.index=element_number;
       }
       else if ((primary_temp) && (p_strcmp(hist_text, primary_temp) == 0))
       {
         gchar* italic_text = g_markup_printf_escaped("<i>%s</i>", string->str);
-			  if( NULL == italic_text) g_printf("NulIMKUp:'%s'\n",string->str);
+			  if( NULL == italic_text) g_fprintf(stderr,"NulIMKUp:'%s'\n",string->str);
         gtk_label_set_markup((GtkLabel*)item_label, italic_text);
         g_free(italic_text);
         h.clip_item=menu_item;
 			  h.element_text=hist_text;
+			  h.wi.index=element_number;
       }
 			if(persistent_history && c->flags &CLIP_TYPE_PERSISTENT){
 				persistent = g_list_prepend(persistent, menu_item);
@@ -1809,7 +1895,7 @@ next_loop:
     menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Edit Clipboard"));
     menu_image = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
     gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
-		g_signal_connect((GObject*)menu_item, "activate", (GCallback)edit_selected, (gpointer)&h);
+		g_signal_connect((GObject*)menu_item, "activate", (GCallback)edit_selected, (gpointer)&h); 
     gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
 		menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Clear"));
 		/* Clear */
@@ -1954,7 +2040,7 @@ static void status_icon_clicked(GtkStatusIcon *status_icon, gpointer user_data)
   /* Control click */
   if (state == GDK_MOD2_MASK+GDK_CONTROL_MASK || state == GDK_CONTROL_MASK)
   {
-		g_printf("Got Ctrl-click\n");
+		g_fprintf(stderr,"Got Ctrl-click\n");
     if (actions_lock == FALSE)
     {
       g_timeout_add(POPUP_DELAY, show_actions_menu, NULL);
@@ -2001,12 +2087,13 @@ void menu_hotkey(char *keystring, gpointer user_data)
 /* Startup calls and initializations */
 static void parcellite_init()
 {
+	int i;
 /* Create clipboard */
   primary = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
   clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
   
 	if(FALSE ==g_thread_supported()){
-		g_printf("g_thread not init!\n");
+		g_fprintf(stderr,"g_thread not init!\n");
 	}
 	clip_lock= g_mutex_new();
 	hist_lock= g_mutex_new();
@@ -2039,10 +2126,8 @@ static void parcellite_init()
   
   /* Bind global keys */
   keybinder_init();
-	keybinder_bind(get_pref_string("phistory_key"), phistory_hotkey, NULL);
-  keybinder_bind(get_pref_string("history_key"), history_hotkey, NULL);
-  keybinder_bind(get_pref_string("actions_key"), actions_hotkey, NULL);
-  keybinder_bind(get_pref_string("menu_key"), menu_hotkey, NULL);
+	for (i=0;NULL != keylist[i].name; ++i)
+		bind_itemkey(keylist[i].name,keylist[i].keyfunc);
   
   /* Create status icon */
   if (show_icon)
@@ -2136,8 +2221,8 @@ int main(int argc, char *argv[])
 	if( NULL != opts->leftovers)g_print("%s\n",opts->leftovers);
 	/**init fifo should set up the fifo and the callback (if we are daemon mode)  */
 		if(opts->primary)	{
-			fifo=init_fifo(FIFO_MODE_PRI|mode);
-			if(fifo->dbg) g_printf("Hit PRI opt!\n");
+			if(NULL == (fifo=init_fifo(FIFO_MODE_PRI|mode))) return 1;
+			if(fifo->dbg) g_fprintf(stderr,"Hit PRI opt!\n");
 			
 			if(PROG_MODE_CLIENT & mode){
 				if(NULL != opts->leftovers){
@@ -2145,7 +2230,7 @@ int main(int argc, char *argv[])
 		      g_free(opts->leftovers);	
 				}
 				
-				if(fifo->dbg) g_printf("checking stdin\n");
+				if(fifo->dbg) g_fprintf(stderr,"checking stdin\n");
 				write_stdin(fifo,FIFO_MODE_PRI);
 				usleep(1000);
 			}
@@ -2158,7 +2243,7 @@ int main(int argc, char *argv[])
 	    g_free(prim_text);
 			
 	  }  else if(opts->clipboard){
-			fifo=init_fifo(FIFO_MODE_CLI|mode);
+			if(NULL == (fifo=init_fifo(FIFO_MODE_CLI|mode))) return 1;
 			
 			if(PROG_MODE_CLIENT & mode){
 				if(NULL != opts->leftovers){
@@ -2177,7 +2262,7 @@ int main(int argc, char *argv[])
 	    g_free(clip_text);
 	  }  else  	{ /*use CLIPBOARD*/
 			GtkClipboard* clip = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-			fifo=init_fifo(FIFO_MODE_NONE|mode);
+			if(NULL == (fifo=init_fifo(FIFO_MODE_NONE|mode))) return 1;
 				/* Copy from unrecognized options */
 			if(PROG_MODE_CLIENT & mode){
 			  if(NULL != opts->leftovers){
