@@ -81,7 +81,7 @@ struct pref_item {
 	gint type;		/**PREF_TYPE_  */
 	gchar *desc; /**shows up in menu  */
 	gchar *tip; /**tooltip */
-	gint sec;	/**cliboard,history, misc,display,hotkeys  */
+	gint sec;	/**clipboard,history, misc,display,hotkeys  */
 	gchar *sig;			 /**signal, if any  */
 	GCallback sfunc; /**function to call  */
 	struct myadj *adj;
@@ -91,6 +91,16 @@ static void check_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void search_toggled(GtkToggleButton *b, gpointer user);
 static gint dbg=0;
 
+/**hot key list, mainly for easy sanity checks.  */
+struct keys keylist[]={
+	{.name="menu_key",.keyval=DEF_MENU_KEY,.keyfunc=(void *)menu_hotkey},
+	{.name="history_key",.keyval=DEF_HISTORY_KEY,.keyfunc=(void *)history_hotkey},
+	{.name="phistory_key",.keyval=DEF_PHISTORY_KEY,.keyfunc=(void *)phistory_hotkey},
+	{.name="actions_key",.keyval=DEF_ACTIONS_KEY,.keyfunc=(void *)actions_hotkey},
+	{.name=NULL,.keyval=NULL,.keyfunc=(void *)0},
+};
+/**must be in same order as above struct array  */
+gchar *def_keyvals[]={ DEF_MENU_KEY,DEF_HISTORY_KEY,DEF_PHISTORY_KEY,DEF_ACTIONS_KEY};
 struct pref_item myprefs[]={
 /**Behaviour  */	
 	/**Clipboards  */
@@ -104,7 +114,7 @@ struct pref_item myprefs[]={
   {.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="history_pos",.type=PREF_TYPE_TOGGLE|PREF_TYPE_SINGLE_LINE,.desc="Position history",.tip="If checked, use X, Y to position the history list",.val=0},
   {.adj=&align_hist_xy,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="history_x",.type=PREF_TYPE_SPIN|PREF_TYPE_SINGLE_LINE,.desc="<b>X</b>",.tip=NULL,.val=1},
   {.adj=&align_hist_xy,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="history_y",.type=PREF_TYPE_SPIN|PREF_TYPE_SINGLE_LINE,.desc="<b>Y</b>",.tip="Position in pixels from the top of the screen",.val=1},
-	{.adj=&align_hist_lim,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="history_limit",.type=PREF_TYPE_SPIN,.desc="Items in history",.tip="Maximun number of cliboard entries to keep",.val=DEF_HISTORY_LIMIT},
+	{.adj=&align_hist_lim,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="history_limit",.type=PREF_TYPE_SPIN,.desc="Items in history",.tip="Maximum number of clipboard entries to keep",.val=DEF_HISTORY_LIMIT},
   {.adj=&align_data_lim,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="data_size",.type=PREF_TYPE_SPIN,.desc="Max Data Size(MB)",.tip="Maximum data size of entire history list",.val=0},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="nop",.type=PREF_TYPE_SPACER,.desc=" ",.tip=NULL},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="automatic_paste",.type=PREF_TYPE_TOGGLE|PREF_TYPE_SINGLE_LINE,.desc="Auto Paste",.tip="If checked, will use xdotool to paste wherever the mouse is.",.val=0},
@@ -112,6 +122,7 @@ struct pref_item myprefs[]={
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="auto_mouse",.type=PREF_TYPE_TOGGLE|PREF_TYPE_SINGLE_LINE,.desc="Mouse",.tip="If checked, will use middle mouse to paste.",.val=1},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="key_input",.type=PREF_TYPE_TOGGLE,.desc="Keyboard Input",.tip="If checked, will emit the history entry via the keyboard entry.",.val=0},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="restore_empty",.type=PREF_TYPE_TOGGLE,.desc="Restore Empty",.tip="If checked, will restore clipboard entry on application exit.",.val=1},
+	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="rc_edit",.type=PREF_TYPE_TOGGLE,.desc="Right Click Edit",.tip="If checked, edit the history item clicked on.",.val=0},
   
   /**Miscellaneous  */  
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sfunc=NULL,.sec=PREF_SEC_MISC,.name=NULL,.type=PREF_TYPE_FRAME,.desc="<b>Miscellaneous</b>",.tip=NULL,.val=0},
@@ -147,10 +158,10 @@ struct pref_item myprefs[]={
 #endif
 	
 /**Action Keys  */
+	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_ACT,.name="menu_key",.type=PREF_TYPE_ENTRY,.desc="Menu key combination",.tip=NULL},	
   {.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_ACT,.name="history_key",.type=PREF_TYPE_ENTRY,.desc="History key combination:",.tip=NULL},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_ACT,.name="phistory_key",.type=PREF_TYPE_ENTRY,.desc="Persistent history key:",.tip=NULL},
   {.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_ACT,.name="actions_key",.type=PREF_TYPE_ENTRY,.desc="Actions key combination:",.tip=NULL},
-  {.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_ACT,.name="menu_key",.type=PREF_TYPE_ENTRY,.desc="Menu key combination",.tip=NULL},	
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_NONE,.name="no_icon",.val=FALSE},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_NONE,.name=NULL,.desc=NULL},
 };
@@ -332,9 +343,59 @@ void bind_itemkey(char *name, void (fhk)(char *, gpointer) )
 		if(dbg)g_printf("pref2:null found for %s\n",name);
 		return;
 	}
-	keybinder_bind(p->cval, fhk, NULL);
+	if(NULL != p->cval && 0 != p->cval)
+		keybinder_bind(p->cval, fhk, NULL);
 }
 
+
+/***************************************************************************/
+/** .
+\n\b Arguments:
+\n\b Returns:
+****************************************************************************/
+void set_key_entry(gchar *name, gchar *val)
+{
+	int i;
+	for (i=0;NULL != keylist[i].name; ++i){
+		if(!g_strcmp0(keylist[i].name,name)){
+			keylist[i].keyval=val;
+			return;
+		}
+	}
+}
+
+/***************************************************************************/
+/** .
+\n\b Arguments:
+\n\b Returns:
+****************************************************************************/
+void set_keys_from_prefs( void )
+{
+	int i,l;
+	gchar *cval;
+	for (i=0;NULL != keylist[i].name; ++i){
+		/**NOTE: do not set up default keys here! User may WANT them null */
+			/** call egg_accelerator_parse_virtual to validate? */
+		set_key_entry(keylist[i].name,get_pref_string(keylist[i].name));
+		/*g_fprintf(stderr,"key '%s' val '%s'\n",keylist[i].name, keylist[i].keyval); */
+	}	
+	/**now go through and make sure we have no duplicates */
+	for (i=0;NULL != keylist[i].name; ++i){
+		if(0 !=  keylist[i].keyval[0]){
+			/**see if it exists elsewhere  */
+			for (l=0;NULL != keylist[l].name; ++l){
+				if(l!=i && 0 != keylist[l].keyval[0]){
+					if(!g_strcmp0(keylist[i].keyval, keylist[l].keyval)) { /**conflict!, delete second  */
+						g_fprintf(stderr,"Error! Hot keys have same key '%s': '%s' and '%s'. Ignoring second entry\n",keylist[i].keyval,keylist[i].name,keylist[l].name);
+						set_key_entry(keylist[l].name,"");
+						set_pref_string(keylist[l].name,"");
+					}
+				}	
+			}	
+		}
+	}	
+	
+}
 /***************************************************************************/
 /** .
 \n\b Arguments:
@@ -343,6 +404,7 @@ void bind_itemkey(char *name, void (fhk)(char *, gpointer) )
 void check_sanity(void)
 {
 	gint32 x,y;
+	gchar *val;
 	postition_history(NULL,&x,&y,NULL, (gpointer)1);
   if(get_pref_int32("history_x")>x) set_pref_int32("history_x",x);
   if(get_pref_int32("history_y")>y) set_pref_int32("history_y",y);
@@ -355,14 +417,9 @@ void check_sanity(void)
 	x=get_pref_int32("ellipsize");
   if ((!x) || (x > 3) || (x < 0))
      set_pref_int32("ellipsize",DEF_ELLIPSIZE);
-	if (NULL == get_pref_string("phistory_key"))
-    set_pref_string("phistory_key",DEF_PHISTORY_KEY);
-  if (NULL == get_pref_string("history_key"))
-    set_pref_string("history_key", DEF_HISTORY_KEY);
-  if (NULL == get_pref_string("actions_key"))
-    set_pref_string("actions_key",DEF_ACTIONS_KEY);
-  if (NULL == get_pref_string("menu_key"))
-    set_pref_string("menu_key",DEF_MENU_KEY);
+	set_keys_from_prefs();
+
+	
 	if(get_pref_int32("persistent_history")){
 	 	if(get_pref_int32("persistent_separate"))
 	 		set_pref_int32("persistent_on_top",0);
@@ -383,11 +440,9 @@ static void apply_preferences()
 {
 	int i;
   /* Unbind the keys before binding new ones */
-	unbind_itemkey("phistory_key",phistory_hotkey);
+	for (i=0;NULL != keylist[i].name; ++i)
+	  unbind_itemkey(keylist[i].name,keylist[i].keyfunc);
 	
-	unbind_itemkey("history_key", history_hotkey);
-  unbind_itemkey("actions_key", actions_hotkey);
-  unbind_itemkey("menu_key", menu_hotkey);
 	
   for (i=0;NULL != myprefs[i].desc; ++i){
 		if(NULL == myprefs[i].name)
@@ -413,10 +468,8 @@ static void apply_preferences()
 	}
   check_sanity();
   /* Bind keys and apply the new history limit */
-	bind_itemkey("phistory_key", phistory_hotkey);
-  bind_itemkey("history_key", history_hotkey);
-  bind_itemkey("actions_key", actions_hotkey);
-  bind_itemkey("menu_key", menu_hotkey);
+	for (i=0;NULL != keylist[i].name; ++i)
+	  bind_itemkey(keylist[i].name,keylist[i].keyfunc);	
   truncate_history();
 }
 
@@ -522,13 +575,11 @@ void read_preferences()
     /* Check for errors and set default values if any */
     check_sanity();
   }
-  else
-  {
-    /* Init default keys on error */
-    set_pref_string("phistory_key",DEF_PHISTORY_KEY);
-    set_pref_string("history_key",DEF_HISTORY_KEY);
-    set_pref_string("actions_key",DEF_ACTIONS_KEY);
-    set_pref_string("menu_key",DEF_MENU_KEY);
+  else  { /* Init default keys on error */
+	  int i;
+		
+		for (i=0;NULL != keylist[i].name; ++i)
+			set_pref_string(keylist[i].name,def_keyvals[i]);
   }
   g_key_file_free(rc_key);
   g_free(rc_file);
@@ -823,7 +874,7 @@ int add_section(int sec, GtkWidget *parent)
 			case PREF_TYPE_FRAME:/**must be first in section, since it sets vbox.  */
 					myprefs[i].w= gtk_frame_new(NULL);
 				  gtk_frame_set_shadow_type((GtkFrame*)	myprefs[i].w, GTK_SHADOW_NONE);
-				  label = gtk_label_new(NULL);							/**<b>myprefs[i].descClipboards</b>"  */
+				  label = gtk_label_new(NULL);							/**<b>myprefs[i].desc  */
 				  gtk_label_set_markup((GtkLabel*)label, myprefs[i].desc);
 				  gtk_frame_set_label_widget((GtkFrame*)	myprefs[i].w, label);
 				  alignment = gtk_alignment_new(0.50, 0.50, 1.0, 1.0);
@@ -930,7 +981,7 @@ void show_preferences(gint tab)
                                                     GTK_STOCK_CANCEL,   GTK_RESPONSE_REJECT,
                                                     GTK_STOCK_OK,       GTK_RESPONSE_ACCEPT, NULL);
   
-  gtk_window_set_icon((GtkWindow*)dialog, gtk_widget_render_icon(dialog, GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU, NULL));
+  gtk_window_set_icon((GtkWindow*)dialog, gtk_widget_render_icon(dialog, GTK_STOCK_PREFERENCES, -1, NULL));
   gtk_window_set_resizable((GtkWindow*)dialog, FALSE);
   
   /* Create notebook */
@@ -1002,7 +1053,8 @@ void show_preferences(gint tab)
   gtk_container_add((GtkContainer*)page_actions, vbox_actions);
   
   /* Build the actions label */
-  label = gtk_label_new(_("Control-click Parcellite\'s tray icon to use actions"));
+  label = gtk_label_new(_("Control-click Parcellite's tray icon to use actions,"));
+	gtk_widget_set_tooltip_text(label, _("Use %% if you want a literal %. See the man page for gnuC printf (man 2 printf) for more information."));
   gtk_label_set_line_wrap((GtkLabel*)label, TRUE);
   gtk_misc_set_alignment((GtkMisc*)label, 0.0, 0.50);
   gtk_box_pack_start((GtkBox*)vbox_actions, label, FALSE, FALSE, 0);
